@@ -1,12 +1,12 @@
 use crate::io::BufMut;
-use crate::postgres::protocol::Encode;
+use crate::postgres::protocol::Write;
 use byteorder::NetworkEndian;
 
 pub struct Sync;
 
-impl Encode for Sync {
+impl Write for Sync {
     #[inline]
-    fn encode(&self, buf: &mut Vec<u8>) {
+    fn write(&self, buf: &mut Vec<u8>) {
         buf.push(b'S');
         buf.put_i32::<NetworkEndian>(4);
     }
